package kr.co.vividnext.sodalive.admin.explorer

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.explorer.section.ExplorerSection
import kr.co.vividnext.sodalive.explorer.section.QExplorerSection.explorerSection
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface AdminExplorerSectionRepository : JpaRepository<ExplorerSection, Long>, AdminExplorerSectionQueryRepository

interface AdminExplorerSectionQueryRepository {
    fun findByTitle(title: String): ExplorerSection?
    fun findAllWithPaging(offset: Long, limit: Long): List<ExplorerSection>
    fun totalCount(): Int
}

class AdminExplorerSectionQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : AdminExplorerSectionQueryRepository {
    override fun findByTitle(title: String): ExplorerSection? {
        return queryFactory
            .selectFrom(explorerSection)
            .where(explorerSection.title.eq(title))
            .fetchFirst()
    }

    override fun findAllWithPaging(offset: Long, limit: Long): List<ExplorerSection> {
        return queryFactory
            .selectFrom(explorerSection)
            .offset(offset)
            .limit(limit)
            .orderBy(explorerSection.orders.asc())
            .fetch()
    }

    override fun totalCount(): Int {
        return queryFactory
            .selectFrom(explorerSection)
            .fetch()
            .size
    }
}
