package kr.co.vividnext.sodalive.audition.applicant

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RequestPart
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@RequestMapping("/audition/applicant")
class AuditionApplicantController(private val service: AuditionApplicantService) {
    @GetMapping
    fun getAuditionApplicantList(
        @RequestParam auditionRoleId: Long,
        @RequestParam sortType: AuditionApplicantSortType,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getAuditionApplicantList(
                auditionRoleId = auditionRoleId,
                sortType = sortType,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )
    }

    @PostMapping
    fun applyAuditionRole(
        @RequestPart("contentFile")
        contentFile: MultipartFile?,
        @RequestPart("request") requestString: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.applyAuditionRole(
                contentFile = contentFile,
                requestString = requestString,
                member = member
            )
        )
    }
}
