package kr.co.vividnext.sodalive.content.category

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.category.QCategoryContent.categoryContent
import org.springframework.data.jpa.repository.JpaRepository

interface CategoryContentRepository : JpaRepository<CategoryContent, Long>, CategoryContentQueryRepository

interface CategoryContentQueryRepository {
    fun findByContentIdAndCategoryId(contentId: Long, categoryId: Long): CategoryContent?
    fun deleteByCategoryId(categoryId: Long)
}

class CategoryContentQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : CategoryContentQueryRepository {
    override fun findByContentIdAndCategoryId(contentId: Long, categoryId: Long): CategoryContent? {
        return queryFactory
            .selectFrom(categoryContent)
            .where(
                categoryContent.content.id.eq(contentId)
                    .and(categoryContent.category.id.eq(categoryId))
            )
            .fetchFirst()
    }

    override fun deleteByCategoryId(categoryId: Long) {
        queryFactory
            .update(categoryContent)
            .set(categoryContent.isActive, false)
            .where(categoryContent.category.id.eq(categoryId))
            .execute()
    }
}
