package kr.co.vividnext.sodalive.content.main.banner

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.main.banner.QAudioContentBanner.audioContentBanner
import kr.co.vividnext.sodalive.content.main.tab.QAudioContentMainTab.audioContentMainTab
import kr.co.vividnext.sodalive.event.QEvent.event
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository

interface AudioContentBannerRepository : JpaRepository<AudioContentBanner, Long>, AudioContentBannerQueryRepository

interface AudioContentBannerQueryRepository {
    fun getAudioContentMainBannerList(tabId: Long, isAdult: Boolean): List<AudioContentBanner>
}

class AudioContentBannerQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : AudioContentBannerQueryRepository {
    override fun getAudioContentMainBannerList(tabId: Long, isAdult: Boolean): List<AudioContentBanner> {
        var where = audioContentBanner.isActive.isTrue

        where = if (tabId == 1L) {
            where.and(audioContentBanner.tab.isNull)
        } else {
            where.and(audioContentBanner.tab.id.eq(tabId))
        }

        if (!isAdult) {
            where = where.and(audioContentBanner.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(audioContentBanner)
            .leftJoin(audioContentBanner.tab, audioContentMainTab)
            .leftJoin(audioContentBanner.event, event)
            .leftJoin(audioContentBanner.creator, member)
            .where(where)
            .orderBy(audioContentBanner.orders.asc())
            .fetch()
    }
}
