package kr.co.vividnext.sodalive.content.main.tab.home

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.member.Member
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/v2/audio-content/main/home")
class AudioContentMainTabHomeController(private val service: AudioContentMainTabHomeService) {
    @GetMapping
    fun fetchContentMainHome(
        @RequestParam("isAdultContentVisible", required = false) isAdultContentVisible: Boolean? = null,
        @RequestParam("contentType", required = false) contentType: ContentType? = null,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        ApiResponse.ok(
            service.fetchData(
                isAdultContentVisible = isAdultContentVisible ?: true,
                contentType = contentType ?: ContentType.ALL,
                member
            )
        )
    }

    @GetMapping("/popular-content-by-creator")
    fun getPopularContentByCreator(
        @RequestParam creatorId: Long,
        @RequestParam("isAdultContentVisible", required = false) isAdultContentVisible: Boolean? = null,
        @RequestParam("contentType", required = false) contentType: ContentType? = null,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        ApiResponse.ok(
            service.getPopularContentByCreator(
                creatorId = creatorId,
                isAdult = member?.auth != null && (isAdultContentVisible ?: true),
                contentType = contentType ?: ContentType.ALL
            )
        )
    }

    @GetMapping("/content/ranking")
    fun getContentRanking(
        @RequestParam("sort-type", required = false) sortType: String? = "매출",
        @RequestParam("isAdultContentVisible", required = false) isAdultContentVisible: Boolean? = null,
        @RequestParam("contentType", required = false) contentType: ContentType? = null,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        ApiResponse.ok(
            service.getContentRanking(
                sortType = sortType ?: "매출",
                isAdultContentVisible = isAdultContentVisible ?: true,
                contentType = contentType ?: ContentType.ALL,
                member
            )
        )
    }
}
