package kr.co.vividnext.sodalive.content.playlist

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.DeleteMapping
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/audio-content/playlist")
class AudioContentPlaylistController(private val service: AudioContentPlaylistService) {
    @PostMapping
    fun createPlaylist(
        @RequestBody request: CreatePlaylistRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.createPlaylist(request, member))
    }

    @PutMapping("/{id}")
    fun updatePlaylist(
        @PathVariable id: Long,
        @RequestBody request: UpdatePlaylistRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.updatePlaylist(playlistId = id, request = request, member = member))
    }

    @DeleteMapping("/{id}")
    fun deletePlaylist(
        @PathVariable id: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.deletePlaylist(playlistId = id, member))
    }

    @GetMapping
    fun getPlaylists(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getPlaylists(member))
    }

    @GetMapping("/{id}")
    fun getPlaylistDetail(
        @PathVariable id: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getPlaylistDetail(playlistId = id, member = member))
    }
}
