package kr.co.vividnext.sodalive.creator.admin.content.category

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@PreAuthorize("hasRole('CREATOR')")
@RequestMapping("/creator-admin/content-category")
class CreatorAdminCategoryController(private val service: CreatorAdminCategoryService) {
    @GetMapping("/search")
    fun searchContentNotInCategory(
        @RequestParam(value = "category_id") categoryId: Long,
        @RequestParam(value = "search_word") searchWord: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.searchContentNotInCategory(
                categoryId = categoryId,
                searchWord = searchWord,
                memberId = member.id!!
            )
        )
    }

    @GetMapping
    fun getContentInCategory(
        @RequestParam(value = "category_id") categoryId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getContentInCategory(
                categoryId = categoryId,
                memberId = member.id!!,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )
    }
}
