package kr.co.vividnext.sodalive.live.signature

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.live.room.donation.LiveRoomDonationResponse
import kr.co.vividnext.sodalive.live.room.donation.QLiveRoomDonationResponse
import kr.co.vividnext.sodalive.live.signature.QSignatureCan.signatureCan
import org.springframework.data.jpa.repository.JpaRepository

interface SignatureCanRepository : JpaRepository<SignatureCan, Long>, SignatureCanQueryRepository

interface SignatureCanQueryRepository {
    fun findImageByCreatorIdAndCan(creatorId: Long, can: Int, imageHost: String, isAdult: Boolean): String?
    fun findByCreatorIdAndCan(creatorId: Long, can: Int, imageHost: String, isAdult: Boolean): LiveRoomDonationResponse?
}

class SignatureCanQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : SignatureCanQueryRepository {
    override fun findImageByCreatorIdAndCan(creatorId: Long, can: Int, imageHost: String, isAdult: Boolean): String? {
        var where = signatureCan.creator.id.eq(creatorId)
            .and(signatureCan.can.eq(can))
            .and(signatureCan.isActive.isTrue)

        if (!isAdult) {
            where = where.and(signatureCan.isAdult.isFalse())
        }

        return queryFactory
            .select(signatureCan.image.prepend("/").prepend(imageHost))
            .from(signatureCan)
            .where(where)
            .orderBy(signatureCan.isAdult.desc())
            .fetchFirst()
    }

    override fun findByCreatorIdAndCan(
        creatorId: Long,
        can: Int,
        imageHost: String,
        isAdult: Boolean
    ): LiveRoomDonationResponse? {
        var where = signatureCan.creator.id.eq(creatorId)
            .and(signatureCan.can.eq(can))
            .and(signatureCan.isActive.isTrue)

        if (!isAdult) {
            where = where.and(signatureCan.isAdult.isFalse())
        }

        return queryFactory
            .select(
                QLiveRoomDonationResponse(
                    signatureCan.image.prepend("/").prepend(imageHost),
                    signatureCan.time
                )
            )
            .from(signatureCan)
            .where(where)
            .orderBy(signatureCan.isAdult.desc())
            .fetchFirst()
    }
}
